package main;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.Collections; 

import design.Block;
import design.Property;
import design.State;
import runtime.IdentifiedState;
import timeSeries.TimeSeriesDatabase;

public class Main {
	private static TimeSeriesDatabase db = null;
	private static int counter =0;

	public static void main(String[] args) {

		setUpDatabase("./lib/Daten_156.csv", false, 0);
		System.out.println("single");
		singleStream("./lib/realStates_156.csv");
		System.out.println("three");
		threeStreams("./lib/realStates_156.csv");
		System.out.println("five");
		fiveStreams("./lib/realStates_156.csv");

		db = null;
		System.out.println("1560 rows");
		setUpDatabase("./lib/Daten_1560.csv", false, 0);
		System.out.println("single");
		singleStream("./lib/realStates_1560.csv");
		System.out.println("three");
		threeStreams("./lib/realStates_1560.csv");
		System.out.println("five");
		fiveStreams("./lib/realStates_1560.csv");

		/*
		// long run 15600
		db = null;
		setUpDatabase("./lib/Daten_1560.csv", true, 9);
		System.out.println("single");
		singleStream("./lib/realStates_1560.csv");
		System.out.println("three");
		threeStreams("./lib/realStates_1560.csv");
		System.out.println("five");
		fiveStreams("./lib/realStates_1560.csv");

		// long run 156000
		db = null;
		setUpDatabase("./lib/Daten_1560.csv", true, 99);
		System.out.println("single");
		singleStream("./lib/realStates_1560.csv");
		System.out.println("three");
		threeStreams("./lib/realStates_1560.csv");
		System.out.println("five");
		fiveStreams("./lib/realStates_1560.csv");

		// long run 1560000
		db = null;
		setUpDatabase("./lib/Daten_1560.csv", true, 900);
		System.out.println("single");
		singleStream("./lib/realStates_1560.csv");
		System.out.println("three");
		threeStreams("./lib/realStates_1560.csv");
		System.out.println("five");
		fiveStreams("./lib/realStates_1560.csv");
		*/

	}

	private static void setUpDatabase(String filenameData, boolean longRun, int timespan) {
		if (db == null) {
			// set up Database and Connection
			db = new TimeSeriesDatabase();
			// set up Data in TSDB
			db.setUpData(filenameData, longRun, timespan);
		}
	}

	private static void singleStream(String filenameReal) {
		// Test State Identification
		Evaluation eval = new Evaluation();
		// SetUp all information about the states in the files
		eval.setUpRealDataSingleStream(filenameReal);

		evaluate(eval, db, setUpDataSingleStream());

	}

	private static void threeStreams(String filenameReal) {
		// Test State Identification
		Evaluation eval = new Evaluation();
		// SetUp all information about the states in the files
		eval.setUpRealDataThreeStreams(filenameReal);

		evaluate(eval, db, setUpDataThreeStreams());

	}

	private static void fiveStreams(String filenameReal) {
		// Test State Identification
		Evaluation eval = new Evaluation();
		// SetUp all information about the states in the files
		eval.setUpRealDataFiveStreams(filenameReal);

		evaluate(eval, db, setUpDataFiveStreams());

	}

	private static void evaluate(Evaluation eval, TimeSeriesDatabase db, Block roboticArm) {
		ArrayList<IdentifiedState> recStates = new ArrayList<IdentifiedState>();
		recStates = eval.testRecognition(db, roboticArm);
		List<EvaluationResult> allResults = new ArrayList<EvaluationResult>();
		PrintWriter pw;
		String dir = "modelTest/";
		String filename = "result"+counter+".csv";
		String path = dir+filename;
		try {
			File f = new File(dir,filename);
			f.createNewFile();
			pw = new PrintWriter(new File(path));
			String row ="";
			Collections.sort(recStates);
			int countProcess =0;
			for (IdentifiedState identifiedState : recStates) {
				if(identifiedState.getName().equals("DriveDown")) {
					countProcess++;	
				}
				row = countProcess+";"+identifiedState.getName()+ ";"+identifiedState.getTs().toString()+"\n";
				pw.write(row);
			}
	        pw.close();
	        counter++;
	    } catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
        // Calculate Precision and Recall
		allResults.addAll(eval.calculatePrecisionRecall(recStates));
	}

	private static Block setUpDataSingleStream() {
		// setUp Information for state DriveDown
		Property p1 = new Property("gp", 1.5);

		List<Property> props1 = new ArrayList<Property>();
		props1.add(p1);
		State s1 = new State("DriveDown", props1);

		// setUp Information for state PickUp
		Property p2 = new Property("gp", -0.4);
		List<Property> props2 = new ArrayList<Property>();
		props2.add(p2);
		State s2 = new State("PickUp", props2);

		// List states
		List<State> states = new ArrayList<State>();
		states.add(s1);
		states.add(s2);

		// SetUp Block
		Block roboticArm = new Block("roboticArm", states);
		return roboticArm;
	}

	private static Block setUpDataThreeStreams() {
		// setUp Information for state DriveDown
		Property p1 = new Property("bp", 0);
		Property p2 = new Property("map", 1.5);
		Property p3 = new Property("gp", 1.5);

		List<Property> props1 = new ArrayList<Property>();
		props1.add(p1);
		props1.add(p2);
		props1.add(p3);
		State s1 = new State("DriveDown", props1);

		// setUp Information for state PickUp
		Property p4 = new Property("bp", 0);
		Property p5 = new Property("map", 1.5);
		Property p6 = new Property("gp", -0.4);

		List<Property> props2 = new ArrayList<Property>();
		props2.add(p4);
		props2.add(p5);
		props2.add(p6);
		State s2 = new State("PickUp", props2);

		// List states
		List<State> states = new ArrayList<State>();
		states.add(s1);
		states.add(s2);

		// SetUp Block
		Block roboticArm = new Block("roboticArm", states);
		return roboticArm;
	}

	private static Block setUpDataFiveStreams() {
		// setUp Information for state DriveDown
		Property p1 = new Property("bp", 0);
		Property p2 = new Property("map", 1.5);
		Property p3 = new Property("gp", 1.5);
		Property p4 = new Property("sap", -0.12);
		Property p5 = new Property("wp", 0);

		List<Property> props1 = new ArrayList<Property>();
		props1.add(p1);
		props1.add(p2);
		props1.add(p3);
		props1.add(p4);
		props1.add(p5);
		State s1 = new State("DriveDown", props1);

		// setUp Information for state PickUp
		Property p6 = new Property("bp", 0);
		Property p7 = new Property("map", 1.5);
		Property p8 = new Property("gp", -0.4);
		Property p9 = new Property("sap", -0.12);
		Property p10 = new Property("wp", 0);
		List<Property> props2 = new ArrayList<Property>();
		props2.add(p6);
		props2.add(p7);
		props2.add(p8);
		props2.add(p9);
		props2.add(p10);
		State s2 = new State("PickUp", props2);

		// List states
		List<State> states = new ArrayList<State>();
		states.add(s1);
		states.add(s2);

		// SetUp Block
		Block roboticArm = new Block("roboticArm", states);
		return roboticArm;
	}

}
